This crate provides a thread-safe, extended version of Cuckoo filter, supporting TTL, LRU and custom counters associated with stored data.
All the limitations of original Cuckoo filter still hold, meaning that all additional data is associated with the fingerprint, which may represent a completely different item. For that reason, it is recommended to only use associated data to manage the lifetime of the items, unless false positives are not problematic in your use-case.
This implementation can be used as a regular Cuckoo filter, without the added features too.
Modules
- config
- This module provides configuration types for
crate::CuckooFilter.
Structs
- Associated
Data - Provides access to data associated with an item in the filter.
- Cuckoo
Filter - Thread-safe cuckoo filter, with support for TTL, LRU and custom counters associated with the stored data.
- Cuckoo
Filter Exporter - Exporter for
crate::CuckooFilter. - Exportable
Random State - Implementation of
ExportableRandomState, similar tostd::hash::RandomState. - Fingerprint
- Represents a fingerprint of an item, stored in the
crate::CuckooFilter. - Insert
Values - Values to store with the fingerprint on insertion.
- Lookup
Values - Values to store with the fingerprint on lookups.
Enums
- Export
Error - Error type for export operations.
- Import
Error - Error type for import operations.
Traits
- Exportable
Build Hasher - A trait representing
BuildHasherinstances that can be exported (e.g. persisted into a file).