pub enum ConfigError {
BucketTooBig,
BucketCountTooBig,
BitCountTooHigh,
BitCountTooLow,
}Expand description
Error type for all configuration options.
Variants
BucketTooBig
Error due to requesting buckets that are too big to represent (requiring over usize::MAX
bytes).
BucketCountTooBig
Error due to requesting bucket count that is too big to represent (requiring over usize::MAX
buckets). This is affected by max_entries and bucket_size.
BitCountTooHigh
Error due to requesting more than 32 bits for any of the fields (fingerprint or associated field).
BitCountTooLow
Error due to requesting 0 bits for a field. If a field is enabled, it should take up at least 1 bit.