pub struct CounterConfig {
pub counter_bits: BitCount,
pub change_on_insert: i32,
pub change_on_lookup: i32,
}Expand description
Configuration for the generic counter field.
Used to define memory used by the generic counter field, also affecting its maximum value.
Examples
use cuckoo_clock::config::CounterConfig;
let ttl_config = CounterConfig {
counter_bits: 5.try_into()?,
..Default::default()
};Fields
counter_bits: BitCountHow many bits are used to represent the generic counter. Larget bit counts allow higher counter values to be represented.
change_on_insert: i32Diff to apply to counter on each insert.
change_on_lookup: i32Diff to apply to counter on each lookup.