pub enum ImportError {
Io(Error),
Config(ConfigError),
NonZeroError(TryFromIntError),
InvalidHasherName {
expected: String,
found: String,
},
InvalidHasherHeader {
found: String,
},
}Expand description
Error type for import operations.
Variants
Io(Error)
Error due to underlying IO error.
Config(ConfigError)
Error due to reading invalid stored configuration.
NonZeroError(TryFromIntError)
Error due to reading a zero value for a field that does not allow zeroes.
InvalidHasherName
Error due to mismatch in hasher names, between the stored and expected hasher.
Fields

expected: StringExpected name for the hasher (defined by the type of hasher for crate::CuckooFilter).
InvalidHasherHeader
Error due to malformed header in the export.