Fingerprint

Struct Fingerprint 

pub struct Fingerprint { /* private fields */ }
Represents a fingerprint of an item, stored in the crate::CuckooFilter.

Generally, fingerprint access should not be needed for regular filter usage, but it can be useful to confirm if the evicted fingerprint matches a known item.

Examples

use cuckoo_clock::{CuckooFilter, Fingerprint, config::CuckooConfiguration};

let filter = CuckooFilter::new_random(CuckooConfiguration::builder(100_000).build()?);

// ... use filter

if let Some(evicted_fp: Fingerprint) = filter.insert("new_item") {
    if evicted_fp.matches_key("known_item", &filter) {
        println!("Known item was evicted!");
    }
}

Implementations

Source

impl Fingerprint

pub fn matches_key<K: Hash + ?Sized, H: BuildHasher>( &self, key: &K, filter: &CuckooFilter<H>, ) -> bool

Compares this fingerprint to a key. This will only produce correct results if the same filter that produced this fingerprint is passed into this method.

Returns true if this fingerprint matches the key.