pub struct CuckooConfiguration { /* private fields */ }Expand description
Configuration for the crate::CuckooFilter.
Used to define main cuckoo filter parameters (capacity, bucket size, fingeprint size, max kicks), as well as additional features (TTL, LRU, generic counter).
Create a new instance using CuckooConfiguration::builder.
Examples
use cuckoo_clock::config::CuckooConfiguration;
let config = CuckooConfiguration::builder(100_000)
.fingerprint_bits(14.try_into()?)
.bucket_size(4.try_into()?)
.max_kicks(8)
.build()?;use cuckoo_clock::config::{CuckooConfiguration, TtlConfig, LruConfig};
let config = CuckooConfiguration::builder(100_000)
.fingerprint_bits(14.try_into()?)
.with_ttl(TtlConfig {
ttl: 10.try_into()?,
ttl_bits: 4.try_into()?
})
.with_lru(LruConfig {
counter_bits: 6.try_into()?,
..Default::default()
})
.bucket_size(4.try_into()?)
.max_kicks(8)
.build()?;Implementations
Sourceimpl CuckooConfiguration
impl CuckooConfiguration
Sourcepub const fn builder(max_entries: usize) -> CuckooConfigurationBuilder
pub const fn builder(max_entries: usize) -> CuckooConfigurationBuilder
Creates a new instance of CuckooConfigurationBuilder with provided maximum number of
entries.
Sourcepub const fn get_configured_memory_usage(&self) -> usize
pub const fn get_configured_memory_usage(&self) -> usize
Returns the memory usage of filter that would be created from this configuration in bytes.
Sourcepub fn compatible_layout(&self, other: Self) -> bool
pub fn compatible_layout(&self, other: Self) -> bool
Checks whether this and other configuration have a compatible layout.
If this is true, filter exported by the other configuration can be imported and used with this configuration. The only compatible configuration changes are generally ones that don’t affect size or count of the buckets and items.