pub struct LruConfig {
pub counter_bits: BitCount,
}Expand description
Configuration for the LRU field.
Used to define memory used by the LRU field, also affecting its maximum value.
Examples
use cuckoo_clock::config::LruConfig;
let ttl_config = LruConfig {
counter_bits: 5.try_into()?
};Fields
counter_bits: BitCountNumber of bits used to represent the LRU counter. Larger bit counts allow more values to be represented, allowing items to “accumulate” higher use counts, which will take longer to age.