CuckooFilterExporter

Struct CuckooFilterExporter 

pub struct CuckooFilterExporter<'a, H: ExportableBuildHasher> { /* private fields */ }
Exporter for crate::CuckooFilter.

This implements Read to allow reading this data into whatever storage required.

Implementations

Source

impl<'a, H: ExportableBuildHasher> CuckooFilterExporter<'a, H>

pub fn write_to(&self, writer: impl Write) -> Result<(), ExportError>

Reads in the entire crate::CuckooFilter state into the provided Write instance.

This will lock buckets one by one and can be run concurrently with other crate::CuckooFilter operations, but it may result in a state which combines old values for some buckets and newer valus for some other buckets. The resulting state should still be valid.

pub fn snapshot(&self) -> Result<Vec<u8>, ExportError>

Reads the entire crate::CuckooFilter state into a Vec.

This can be useful if writing to slower Write interfaces, to ensure less changes are made while exporting is in progress.